@@ -22,7 +22,7 @@ __all__ = [
     "neutral",
     "load_h5",
 ]
-__author__ = ["Jin Hyun Cheong"]
+__author__ = ["Jin Hyun Cheong, Tiankang Xie"]
 
 
 import os, math, pywt, pickle, h5py
@@ -98,8 +98,11 @@ openface_3d_landmark_columns = (
 )
 
 jaanet_AU_list = [1, 2, 4, 6, 7, 10, 12, 14, 15, 17, 23, 24]
+RF_AU_list = [1,2,4,5,6,7,9,10,11,12,14,15,17,20,23,24,25,26,28,43]
 jaanet_AU_presence = [f"AU" + str(i).zfill(2) for i in jaanet_AU_list]
 jaanet_AU_presence.sort()
+RF_AU_presence = [f"AU" + str(i).zfill(2) for i in RF_AU_list]
+RF_AU_presence.sort()
 
 openface_AU_list = [1, 2, 4, 5, 6, 7, 9, 10, 12, 14, 15, 17, 20, 23, 25, 26, 45]
 openface_AU_intensity = [f"AU" + str(i).zfill(2) + "_r" for i in openface_AU_list]
@@ -247,7 +250,10 @@ def read_feat(fexfile):
     """This function reads files extracted using the Detector from the Feat package.
 
     Args:
-        fexfile: [description]
+        fexfile: Path to facial expression file.
+
+    Returns:
+        Fex of processed facial expressions
     """
     d = pd.read_csv(fexfile)
     return feat.Fex(
@@ -267,15 +273,10 @@ def read_facet(facetfile, features=None, raw=False, sampling_freq=None):
 
     Args:
         facetfile: iMotions-FACET file. Files from iMotions 5, 6, and 7 have been tested and supported
-        features: If a list of iMotion-FACET column names are passed, those are returned.
-                Otherwise, default columns are returned in the following format:['Timestamp','FaceRectX','FaceRectY','FaceRectWidth','FaceRectHeight',
-                'Joy','Anger','Surprise','Fear','Contempt', 'Disgust','Sadness','Confusion','Frustration',
-                'Neutral','Positive','Negative','AU1','AU2', 'AU4','AU5','AU6','AU7','AU9','AU10',
-                'AU12','AU14','AU15','AU17','AU18','AU20', 'AU23','AU24','AU25','AU26','AU28','AU43',
-                'Yaw', 'Pitch', 'Roll'].
-                Note that these column names are different from the original files which has ' Evidence', ' Degrees' appended to each column.
+        features: If a list of iMotion-FACET column names are passed, those are returned. Otherwise, default columns are returned in the following format:['Timestamp','FaceRectX','FaceRectY','FaceRectWidth','FaceRectHeight', 'Joy','Anger','Surprise','Fear','Contempt', 'Disgust','Sadness','Confusion','Frustration', 'Neutral','Positive','Negative','AU1','AU2', 'AU4','AU5','AU6','AU7','AU9','AU10', 'AU12','AU14','AU15','AU17','AU18','AU20', 'AU23','AU24','AU25','AU26','AU28','AU43', 'Yaw', 'Pitch', 'Roll']. Note that these column names are different from the original files which has ' Evidence', ' Degrees' appended to each column.
         raw (default=False): Set to True to return all columns without processing.
         sampling_freq: sampling frequency to pass to Fex
+
     Returns:
         dataframe of processed facial expressions
     """
@@ -427,11 +428,14 @@ def read_openface(openfacefile, features=None):
 
 def read_affectiva(affectivafile, orig_cols=False):
     """
-    This function reads in affectiva file processed through
-    the https://github.com/cosanlab/affectiva-api-app.
+    This function reads in affectiva file processed through the https://github.com/cosanlab/affectiva-api-app.
+
     Args:
         affectivafile: file to read
         orig_cols: If True, convert original colnames to FACS names
+
+    Returns:
+        Fex of processed facial expressions
     """
     d = pd.read_json(affectivafile, lines=True)
     rep_dict = {
@@ -486,14 +490,13 @@ def read_affectiva(affectivafile, orig_cols=False):
 
 
 def wavelet(freq, num_cyc=3, sampling_freq=30.0):
-    """Create a complex Morlet wavelet by windowing a cosine function by a
-    Gaussian. All formulae taken from Cohen, 2014 Chaps 12 + 13
+    """Create a complex Morlet wavelet.
+    
+    Creates a complex Morlet wavelet by windowing a cosine function by a Gaussian. All formulae taken from Cohen, 2014 Chaps 12 + 13
 
     Args:
         freq: (float) desired frequence of wavelet
-        num_cyc: (float) number of wavelet cycles/gaussian taper. Note that
-                 smaller cycles give greater temporal precision and that larger
-                 values give greater frequency precision; (default: 3)
+        num_cyc: (float) number of wavelet cycles/gaussian taper. Note that smaller cycles give greater temporal precision and that larger values give greater frequency precision; (default: 3)
         sampling_freq: (float) sampling frequency of original signal.
 
     Returns:
@@ -513,20 +516,15 @@ def wavelet(freq, num_cyc=3, sampling_freq=30.0):
 
 
 def calc_hist_auc(vals, hist_range=None):
-    """
-    This function follows the bag of temporal feature analysis as described in
-    Bartlett, M. S., Littlewort, G. C., Frank, M. G., & Lee, K. (2014).
-    Automatic decoding of facial movements reveals deceptive pain expressions.
-    Current Biology, 24(7), 738-743.
-    The function receives convolved data, squares the values,
-    finds 0 crossings to calculate the AUC(area under the curve)
-    and generates a 6 exponentially-spaced-bin histogram for each data.
+    """Calculate histogram area under the curve.
+
+    This function follows the bag of temporal feature analysis as described in Bartlett, M. S., Littlewort, G. C., Frank, M. G., & Lee, K. (2014). Automatic decoding of facial movements reveals deceptive pain expressions. Current Biology, 24(7), 738-743. The function receives convolved data, squares the values, finds 0 crossings to calculate the AUC(area under the curve) and generates a 6 exponentially-spaced-bin histogram for each data.
 
     Args:
+        vals: 
 
     Returns:
-
-
+        Series of histograms
     """
     # Square values
     vals = [elem ** 2 if elem > 0 else -1 * elem ** 2 for elem in vals]
@@ -642,16 +640,15 @@ neutral = np.array(
 
 
 def registration(face_lms, neutral=neutral, method="fullface"):
-    """
+    """Register faces to a neutral face.
+
     Affine registration of face landmarks to neutral face.
 
     Args:
-        face_lms(array): face landmarks to register with shape (n,136).
-                         Columns 0~67 are x coordinates and 68~136 are y coordinates
+        face_lms(array): face landmarks to register with shape (n,136). Columns 0~67 are x coordinates and 68~136 are y coordinates
         neutral(array): target neutral face array that face_lm will be registered
-        method(str or list): If string, register to all landmarks ('fullface', default),
-                    or inner parts of face nose,mouth,eyes, and brows ('inner').
-                    If list, pass landmarks to register to e.g. [27, 28, 29, 30, 36, 39, 42, 45]
+        method(str or list): If string, register to all landmarks ('fullface', default), or inner parts of face nose,mouth,eyes, and brows ('inner'). If list, pass landmarks to register to e.g. [27, 28, 29, 30, 36, 39, 42, 45]
+
     Return:
         registered_lms: registered landmarks in shape (n,136)
     """
@@ -688,12 +685,13 @@ def registration(face_lms, neutral=neutral, method="fullface"):
 
 
 def convert68to49(points):
-    """
-    Function slightly modified from https://github.com/D-X-Y/landmark-detection/blob/7bc7a5dbdbda314653124a4596f3feaf071e8589/SAN/lib/datasets/dataset_utils.py#L169
-    to fit pytorch tensors.
-    Converts 68 point landmarks to 49 point landmarks
+    """Convert landmark form 68 to 49
+
+    Function slightly modified from https://github.com/D-X-Y/landmark-detection/blob/7bc7a5dbdbda314653124a4596f3feaf071e8589/SAN/lib/datasets/dataset_utils.py#L169 to fit pytorch tensors. Converts 68 point landmarks to 49 point landmarks
+    
     Args:
         points: landmark points of shape (2,68) or (3,68)
+
     Return:
         cpoints: converted 49 landmark points of shape (2,49)
     """
@@ -748,15 +746,18 @@ class BBox(object):
 
 
 def drawLandmark(img, bbox, landmark):
-    # https://github.com/cunjian/pytorch_face_landmark/
-    """
-    Input:
-    - img: gray or RGB
-    - bbox: type of BBox
-    - landmark: reproject landmark of (5L, 2L)
-    Output:
-    - img marked with landmark and bbox
-    """
+    """Draws face bounding box and landmarks.
+
+    From https://github.com/cunjian/pytorch_face_landmark/
+
+    Args:
+        img ([type]): gray or RGB
+        bbox ([type]): type of BBox
+        landmark ([type]): reproject landmark of (5L, 2L)
+
+    Returns:
+        img marked with landmark and bbox
+    """ 
     img_ = img.copy()
     cv2.rectangle(
         img_, (bbox.left, bbox.top), (bbox.right, bbox.bottom), (0, 0, 255), 2
@@ -767,16 +768,95 @@ def drawLandmark(img, bbox, landmark):
 
 
 def drawLandmark_multiple(img, bbox, landmark):
-    # https://github.com/cunjian/pytorch_face_landmark/
-    """
-    Input:
-    - img: gray or RGB
-    - bbox: type of BBox
-    - landmark: reproject landmark of (5L, 2L)
-    Output:
-    - img marked with landmark and bbox
-    """
+    """Draw multiple landmarks.
+
+    From https://github.com/cunjian/pytorch_face_landmark/
+    
+    Args:
+        img ([type]): gray or RGB
+        bbox ([type]): type of BBox
+        landmark ([type]): reproject landmark of (5L, 2L)
+
+    Returns:
+        img marked with landmark and bbox
+    """    
     cv2.rectangle(img, (bbox.left, bbox.top), (bbox.right, bbox.bottom), (0, 0, 255), 2)
     for x, y in landmark:
         cv2.circle(img, (int(x), int(y)), 2, (0, 255, 0), -1)
     return img
+
+def padding(img, expected_size):
+    """
+    DOCUMENTATION GOES HERE
+    """
+    desired_size = expected_size
+    delta_width = desired_size - img.size[0]
+    delta_height = desired_size - img.size[1]
+    pad_width = delta_width // 2
+    pad_height = delta_height // 2
+    padding = (pad_width, pad_height, delta_width - pad_width, delta_height - pad_height)
+    return ImageOps.expand(img, padding)
+
+
+def resize_with_padding(img, expected_size):
+    """
+    DOCUMENTATION GOES HERE
+    """
+    img.thumbnail((expected_size[0], expected_size[1]))
+    # print(img.size)
+    delta_width = expected_size[0] - img.size[0]
+    delta_height = expected_size[1] - img.size[1]
+    pad_width = delta_width // 2
+    pad_height = delta_height // 2
+    padding = (pad_width, pad_height, delta_width - pad_width, delta_height - pad_height)
+    return ImageOps.expand(img, padding)
+
+
+def align_face_68pts(img, img_land, box_enlarge, img_size=112):
+    """Performs affine transformation to align the images by eyes.
+
+    Performs affine alignment including eyes. 
+    
+    Args:
+        img: gray or RGB
+        img_land: 68 system flattened landmarks, shape:(136) 
+        box_enlarge: relative size of face on the image. Smaller value indicate larger proportion
+        img_size = output image size
+    Return:
+        aligned_img: the aligned image
+        new_land: the new landmarks 
+    """
+    leftEye0 = (img_land[2 * 36] + img_land[2 * 37] + img_land[2 * 38] + img_land[2 * 39] + img_land[2 * 40] +
+                img_land[2 * 41]) / 6.0
+    leftEye1 = (img_land[2 * 36 + 1] + img_land[2 * 37 + 1] + img_land[2 * 38 + 1] + img_land[2 * 39 + 1] +
+                img_land[2 * 40 + 1] + img_land[2 * 41 + 1]) / 6.0
+    rightEye0 = (img_land[2 * 42] + img_land[2 * 43] + img_land[2 * 44] + img_land[2 * 45] + img_land[2 * 46] +
+                img_land[2 * 47]) / 6.0
+    rightEye1 = (img_land[2 * 42 + 1] + img_land[2 * 43 + 1] + img_land[2 * 44 + 1] + img_land[2 * 45 + 1] +
+                img_land[2 * 46 + 1] + img_land[2 * 47 + 1]) / 6.0
+    deltaX = (rightEye0 - leftEye0)
+    deltaY = (rightEye1 - leftEye1)
+    l = math.sqrt(deltaX * deltaX + deltaY * deltaY)
+    sinVal = deltaY / l
+    cosVal = deltaX / l
+    mat1 = np.mat([[cosVal, sinVal, 0], [-sinVal, cosVal, 0], [0, 0, 1]])
+    mat2 = np.mat([[leftEye0, leftEye1, 1], [rightEye0, rightEye1, 1], [img_land[2 * 30], img_land[2 * 30 + 1], 1],
+                [img_land[2 * 48], img_land[2 * 48 + 1], 1], [img_land[2 * 54], img_land[2 * 54 + 1], 1]])
+    mat2 = (mat1 * mat2.T).T
+    cx = float((max(mat2[:, 0]) + min(mat2[:, 0]))) * 0.5
+    cy = float((max(mat2[:, 1]) + min(mat2[:, 1]))) * 0.5
+    if (float(max(mat2[:, 0]) - min(mat2[:, 0])) > float(max(mat2[:, 1]) - min(mat2[:, 1]))):
+        halfSize = 0.5 * box_enlarge * float((max(mat2[:, 0]) - min(mat2[:, 0])))
+    else:
+        halfSize = 0.5 * box_enlarge * float((max(mat2[:, 1]) - min(mat2[:, 1])))
+    scale = (img_size - 1) / 2.0 / halfSize
+    mat3 = np.mat([[scale, 0, scale * (halfSize - cx)], [0, scale, scale * (halfSize - cy)], [0, 0, 1]])
+    mat = mat3 * mat1
+    aligned_img = cv2.warpAffine(img, mat[0:2, :], (img_size, img_size), cv2.INTER_LINEAR, borderValue=(128, 128, 128))
+    land_3d = np.ones((int(len(img_land)/2), 3))
+    land_3d[:, 0:2] = np.reshape(np.array(img_land), (int(len(img_land)/2), 2))
+    mat_land_3d = np.mat(land_3d)
+    new_land = np.array((mat * mat_land_3d.T).T)
+    new_land = np.array(list(zip(new_land[:,0], new_land[:,1]))).astype(int)
+
+    return aligned_img, new_land
